@@ -5,17 +5,17 @@
 use crate::ule::*;
 use alloc::boxed::Box;
 use alloc::vec::Vec;
-use components::SliceComponents;
 use core::fmt::{self, Display};
 use core::ops::Index;
 
-pub(crate) mod components;
+pub(crate) mod borrowed;
 pub(crate) mod owned;
 #[cfg(feature = "serde")]
 mod serde;
 mod ule;
 
-use owned::VarZeroVecOwned;
+pub use borrowed::VarZeroVecBorrowed;
+pub use owned::VarZeroVecOwned;
 pub use ule::VarZeroVecULE;
 
 /// A zero-copy vector for variable-width types.
@@ -27,7 +27,7 @@ pub use ule::VarZeroVecULE;
 /// `T` must implement [`VarULE`], which is already implemented for [`str`] and `[T]` where
 /// `T` implements [`ULE`]. It is also implemented on `VarZeroVecULE<T>` for nesting.
 ///
-/// `VarZeroVec<T>` behaves much like [`std::borrow::Cow`], where it can be constructed from owned data
+/// `VarZeroVec<T>` behaves much like [`Cow`](alloc::borrow::Cow), where it can be constructed from owned data
 /// but can also borrow from some buffer.
 ///
 /// # How it Works
@@ -105,37 +105,47 @@ pub use ule::VarZeroVecULE;
 /// for more information.
 ///
 /// [`ule`]: crate::ule
-#[derive(Clone)]
-pub struct VarZeroVec<'a, T: ?Sized>(VarZeroVecInner<'a, T>);
-
-/// Implementation details of VarZeroVec
-///
-/// Should not be made public.
-///
-/// Safety invariant: Borrowed() must have the appropriate
-/// format:
-///
-/// The format of the slice is conceptually:
-/// ```rust,ignore
-/// struct SliceFormat {
-///     len: u32::ULE,
-///     lens: [u32::ULE],
-///     things: [u8]
-/// }
-/// ```
-///
-/// `len` is the length of `lens`, and each element of `lens` is the starting
-/// index of the thing in `things`.
-///
-/// The actual implementation details of this can be found in the `components` module
-#[derive(Clone)]
-enum VarZeroVecInner<'a, T: ?Sized> {
+pub enum VarZeroVec<'a, T: ?Sized> {
+    /// An allocated VarZeroVec, allowing for mutations.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use zerovec::VarZeroVec;
+    ///
+    /// let mut vzv = VarZeroVec::<str>::default();
+    /// vzv.make_mut().push("foo");
+    /// vzv.make_mut().push("bar");
+    /// assert!(matches!(vzv, VarZeroVec::Owned(_)));
+    /// ```
     Owned(VarZeroVecOwned<T>),
-    /// This is *basically* an `&'a [u8]` to a zero copy buffer, but split out into
-    /// the buffer components. Logically this is capable of behaving as
-    /// a `&'a [T::VarULE]`, but since `T::VarULE` is unsized that type does not actually
-    /// exist
-    Borrowed(SliceComponents<'a, T>),
+    /// A borrowed VarZeroVec, requiring no allocations.
+    ///
+    /// If a mutating operation is invoked on VarZeroVec, the Borrowed is converted to Owned.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use zerovec::VarZeroVec;
+    ///
+    /// let bytes = &[
+    ///     4, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,  3, 0, 0, 0,
+    ///     6, 0, 0, 0, 119, 207, 137, 230, 150, 135, 240, 145, 132, 131,
+    /// ];
+    ///
+    /// let vzv: VarZeroVec<str> = VarZeroVec::parse_byte_slice(bytes).unwrap();
+    /// assert!(matches!(vzv, VarZeroVec::Borrowed(_)));
+    /// ```
+    Borrowed(VarZeroVecBorrowed<'a, T>),
+}
+
+impl<'a, T: ?Sized> Clone for VarZeroVec<'a, T> {
+    fn clone(&self) -> Self {
+        match *self {
+            VarZeroVec::Owned(ref o) => o.clone().into(),
+            VarZeroVec::Borrowed(b) => b.into(),
+        }
+    }
 }
 
 #[derive(Clone, Debug)]
@@ -170,25 +180,47 @@ impl<E> From<E> for VarZeroVecError<E> {
     }
 }
 
-impl<'a, T: ?Sized> From<VarZeroVecInner<'a, T>> for VarZeroVec<'a, T> {
-    #[inline]
-    fn from(other: VarZeroVecInner<'a, T>) -> Self {
-        Self(other)
-    }
-}
-
 impl<'a, T: ?Sized> From<VarZeroVecOwned<T>> for VarZeroVec<'a, T> {
     #[inline]
     fn from(other: VarZeroVecOwned<T>) -> Self {
-        VarZeroVecInner::Owned(other).into()
+        VarZeroVec::Owned(other)
+    }
+}
+
+impl<'a, T: ?Sized> From<VarZeroVecBorrowed<'a, T>> for VarZeroVec<'a, T> {
+    fn from(other: VarZeroVecBorrowed<'a, T>) -> Self {
+        VarZeroVec::Borrowed(other)
+    }
+}
+
+impl<'a, T: ?Sized + VarULE> From<VarZeroVec<'a, T>> for VarZeroVecOwned<T> {
+    #[inline]
+    fn from(other: VarZeroVec<'a, T>) -> Self {
+        match other {
+            VarZeroVec::Owned(o) => o,
+            VarZeroVec::Borrowed(b) => b.into(),
+        }
+    }
+}
+
+impl<T: VarULE + ?Sized> Default for VarZeroVec<'_, T> {
+    fn default() -> Self {
+        Self::new()
     }
 }
 
 impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
-    fn get_components<'b>(&'b self) -> SliceComponents<'b, T> {
-        match self.0 {
-            VarZeroVecInner::Owned(ref owned) => owned.get_components(),
-            VarZeroVecInner::Borrowed(components) => components,
+    /// Construct a new empty [`VarZeroVec`]
+    #[inline]
+    pub fn new() -> Self {
+        VarZeroVecOwned::new().into()
+    }
+
+    /// Obtain a [`VarZeroVecBorrowed`] borrowing from the internal buffer
+    pub fn as_borrowed<'b>(&'b self) -> VarZeroVecBorrowed<'b, T> {
+        match self {
+            VarZeroVec::Owned(ref owned) => owned.as_borrowed(),
+            VarZeroVec::Borrowed(ref borrowed) => *borrowed,
         }
     }
 
@@ -210,7 +242,7 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     /// # Ok::<(), VarZeroVecError<Utf8Error>>(())
     /// ```
     pub fn len(&self) -> usize {
-        self.get_components().len()
+        self.as_borrowed().len()
     }
 
     /// Returns `true` if the vector contains no elements.
@@ -230,7 +262,7 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     /// # Ok::<(), VarZeroVecError<Utf8Error>>(())
     /// ```
     pub fn is_empty(&self) -> bool {
-        self.get_components().is_empty()
+        self.as_borrowed().is_empty()
     }
 
     /// Parse a VarZeroVec from a slice of the appropriate format
@@ -259,12 +291,12 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     pub fn parse_byte_slice(slice: &'a [u8]) -> Result<Self, ParseErrorFor<T>> {
         if slice.is_empty() {
             // does not allocate
-            return Ok(VarZeroVecInner::Owned(VarZeroVecOwned::new()).into());
+            return Ok(VarZeroVec::Owned(VarZeroVecOwned::new()));
         }
 
-        let components = SliceComponents::<T>::parse_byte_slice(slice)?;
+        let borrowed = VarZeroVecBorrowed::<T>::parse_byte_slice(slice)?;
 
-        Ok(VarZeroVecInner::Borrowed(components).into())
+        Ok(VarZeroVec::Borrowed(borrowed))
     }
 
     /// Obtain an iterator over VarZeroVec's elements
@@ -289,7 +321,7 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     /// # Ok::<(), VarZeroVecError<Utf8Error>>(())
     /// ```
     pub fn iter<'b: 'a>(&'b self) -> impl Iterator<Item = &'b T> {
-        self.get_components().iter()
+        self.as_borrowed().iter()
     }
 
     /// Get one of VarZeroVec's elements, returning None if the index is out of bounds
@@ -315,7 +347,7 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     /// # Ok::<(), VarZeroVecError<Utf8Error>>(())
     /// ```
     pub fn get(&self, idx: usize) -> Option<&T> {
-        self.get_components().get(idx)
+        self.as_borrowed().get(idx)
     }
 
     /// Convert this into a mutable vector of the owned `T` type, cloning if necessary.
@@ -347,12 +379,12 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     //
     // This function is crate-public for now since we don't yet want to stabilize
     // the internal implementation details
-    pub(crate) fn make_mut(&mut self) -> &mut VarZeroVecOwned<T> {
-        match self.0 {
-            VarZeroVecInner::Owned(ref mut vec) => vec,
-            VarZeroVecInner::Borrowed(components) => {
-                let new_self = VarZeroVecOwned::from_components(components).into();
-                *self = new_self;
+    pub fn make_mut(&mut self) -> &mut VarZeroVecOwned<T> {
+        match self {
+            VarZeroVec::Owned(ref mut vec) => vec,
+            VarZeroVec::Borrowed(ref borrowed) => {
+                let new_self = VarZeroVecOwned::from_borrowed(*borrowed);
+                *self = new_self.into();
                 // recursion is limited since we are guaranteed to hit the Owned branch
                 self.make_mut()
             }
@@ -380,8 +412,8 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     /// ```
     pub fn into_owned(mut self) -> VarZeroVec<'static, T> {
         self.make_mut();
-        match self.0 {
-            VarZeroVecInner::Owned(vec) => vec.into(),
+        match self {
+            VarZeroVec::Owned(vec) => vec.into(),
             _ => unreachable!(),
         }
     }
@@ -397,21 +429,16 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
 
     /// Obtain an owned `Vec<Box<T>>` out of this
     pub fn to_vec(&self) -> Vec<Box<T>> {
-        self.get_components().to_vec()
-    }
-
-    /// Get a [`VarZeroVec`] that borrows from this one
-    pub fn as_borrowed<'b>(&'b self) -> VarZeroVec<'b, T> {
-        VarZeroVecInner::Borrowed(self.get_components()).into()
+        self.as_borrowed().to_vec()
     }
 
     /// Obtain the internal encoded slice
     ///
     /// This can be passed back to [`Self::parse_byte_slice()`]
     pub fn get_encoded_slice(&self) -> &[u8] {
-        match self.0 {
-            VarZeroVecInner::Owned(ref vec) => vec.entire_slice(),
-            VarZeroVecInner::Borrowed(vec) => vec.entire_slice(),
+        match self {
+            VarZeroVec::Owned(ref vec) => vec.entire_slice(),
+            VarZeroVec::Borrowed(vec) => vec.entire_slice(),
         }
     }
 
@@ -438,16 +465,16 @@ impl<'a, T: VarULE + ?Sized> VarZeroVec<'a, T> {
     /// ```
     ///
     pub fn get_serializable_bytes<A: custom::EncodeAsVarULE<T>>(elements: &[A]) -> Option<Vec<u8>> {
-        components::get_serializable_bytes(elements)
+        borrowed::get_serializable_bytes(elements)
     }
 
     /// Return whether the [`VarZeroVec`] is operating on owned or borrowed
     /// data. [`VarZeroVec::into_owned()`] and [`VarZeroVec::make_mut()`] can
     /// be used to force it into an owned type
     pub fn is_owned(&self) -> bool {
-        match self.0 {
-            VarZeroVecInner::Owned(..) => true,
-            VarZeroVecInner::Borrowed(..) => false,
+        match self {
+            VarZeroVec::Owned(..) => true,
+            VarZeroVec::Borrowed(..) => false,
         }
     }
 }
@@ -481,7 +508,7 @@ where
     /// [`binary_search`]: https://doc.rust-lang.org/std/primitive.slice.html#method.binary_search
     #[inline]
     pub fn binary_search(&self, x: &T) -> Result<usize, usize> {
-        self.get_components().binary_search(x)
+        self.as_borrowed().binary_search(x)
     }
 }
 
