@@ -4,16 +4,18 @@
 
 use super::VarZeroVec;
 use crate::ule::*;
+use alloc::boxed::Box;
+use alloc::vec::Vec;
+use core::fmt;
+use core::marker::PhantomData;
 use serde::de::{self, Deserialize, Deserializer, SeqAccess, Visitor};
-use serde::ser::{self, Serialize, SerializeSeq, Serializer};
-use std::fmt;
-use std::marker::PhantomData;
+use serde::ser::{Serialize, SerializeSeq, Serializer};
 
-struct VarZeroVecVisitor<T> {
-    marker: PhantomData<fn() -> T>,
+struct VarZeroVecVisitor<T: ?Sized> {
+    marker: PhantomData<fn() -> Box<T>>,
 }
 
-impl<T> Default for VarZeroVecVisitor<T> {
+impl<T: ?Sized> Default for VarZeroVecVisitor<T> {
     fn default() -> Self {
         Self {
             marker: PhantomData,
@@ -23,8 +25,8 @@ impl<T> Default for VarZeroVecVisitor<T> {
 
 impl<'de, T> Visitor<'de> for VarZeroVecVisitor<T>
 where
-    T: 'de + Deserialize<'de> + AsVarULE,
-    <<T as AsVarULE>::VarULE as VarULE>::Error: fmt::Display,
+    T: VarULE + ?Sized,
+    Box<T>: Deserialize<'de>,
 {
     type Value = VarZeroVec<'de, T>;
 
@@ -36,30 +38,30 @@ where
     where
         E: de::Error,
     {
-        VarZeroVec::try_from_bytes(bytes).map_err(de::Error::custom)
+        VarZeroVec::parse_byte_slice(bytes).map_err(de::Error::custom)
     }
 
     fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
     where
         A: SeqAccess<'de>,
     {
-        let mut vec: Vec<T> = if let Some(capacity) = seq.size_hint() {
+        let mut vec: Vec<Box<T>> = if let Some(capacity) = seq.size_hint() {
             Vec::with_capacity(capacity)
         } else {
             Vec::new()
         };
-        while let Some(value) = seq.next_element::<T>()? {
+        while let Some(value) = seq.next_element::<Box<T>>()? {
             vec.push(value);
         }
-        Ok(vec.into())
+        Ok((&*vec).into())
     }
 }
 
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
 impl<'de, 'a, T> Deserialize<'de> for VarZeroVec<'a, T>
 where
-    T: 'de + Deserialize<'de> + AsVarULE,
-    <<T as AsVarULE>::VarULE as VarULE>::Error: fmt::Display,
+    T: VarULE + ?Sized,
+    Box<T>: Deserialize<'de>,
     'de: 'a,
 {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
@@ -78,7 +80,7 @@ where
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
 impl<T> Serialize for VarZeroVec<'_, T>
 where
-    T: Serialize + AsVarULE + Clone,
+    T: Serialize + VarULE + ?Sized,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -87,24 +89,11 @@ where
         if serializer.is_human_readable() {
             let mut seq = serializer.serialize_seq(Some(self.len()))?;
             for value in self.iter() {
-                // In the case of T=String this creates an unnecessary
-                // allocation just to throw it away, but we cannot use this guarantee
-                // for all T. We could potentially add a `serialize_unaligned_element`
-                // method to AsVarULE, but since serialization performance is not
-                // critical, this is currently not done.
-                seq.serialize_element(&T::from_unaligned(value))?;
+                seq.serialize_element(value)?;
             }
             seq.end()
-        } else if let Some(slice) = self.get_slice_for_borrowed() {
-            serializer.serialize_bytes(slice)
         } else {
-            // This creates an additional Vec allocation to enable code reuse of
-            // VarZeroVec::to_vec()'s. The alternative is to write a different
-            // implementation of get_serializable_bytes() which enables us to pull
-            // out the byte buffer components bit by bit and use serialize_seq + serialize_element
-            let vec = VarZeroVec::get_serializable_bytes(&self.to_vec())
-                .ok_or_else(|| ser::Error::custom("VarZeroVec too large to be serialized"))?;
-            serializer.serialize_bytes(&vec)
+            serializer.serialize_bytes(self.get_encoded_slice())
         }
     }
 }
@@ -134,28 +123,28 @@ mod test {
     ];
     #[test]
     fn test_serde_json() {
-        let zerovec_orig: VarZeroVec<String> = VarZeroVec::try_from_bytes(BYTES).expect("parse");
+        let zerovec_orig: VarZeroVec<str> = VarZeroVec::parse_byte_slice(BYTES).expect("parse");
         let json_str = serde_json::to_string(&zerovec_orig).expect("serialize");
         assert_eq!(JSON_STR, json_str);
         // VarZeroVec should deserialize from JSON to either Vec or VarZeroVec
-        let vec_new: Vec<String> =
+        let vec_new: Vec<Box<str>> =
             serde_json::from_str(&json_str).expect("deserialize from buffer to Vec");
         assert_eq!(zerovec_orig.to_vec(), vec_new);
-        let zerovec_new: VarZeroVec<String> =
+        let zerovec_new: VarZeroVec<str> =
             serde_json::from_str(&json_str).expect("deserialize from buffer to VarZeroVec");
         assert_eq!(zerovec_orig.to_vec(), zerovec_new.to_vec());
-        assert!(zerovec_new.get_slice_for_borrowed().is_none());
+        assert!(zerovec_new.is_owned());
     }
 
     #[test]
     fn test_serde_bincode() {
-        let zerovec_orig: VarZeroVec<String> = VarZeroVec::try_from_bytes(BYTES).expect("parse");
+        let zerovec_orig: VarZeroVec<str> = VarZeroVec::parse_byte_slice(BYTES).expect("parse");
         let bincode_buf = bincode::serialize(&zerovec_orig).expect("serialize");
         assert_eq!(BINCODE_BUF, bincode_buf);
-        let zerovec_new: VarZeroVec<String> =
+        let zerovec_new: VarZeroVec<str> =
             bincode::deserialize(&bincode_buf).expect("deserialize from buffer to VarZeroVec");
         assert_eq!(zerovec_orig.to_vec(), zerovec_new.to_vec());
-        assert!(zerovec_new.get_slice_for_borrowed().is_some());
+        assert!(!zerovec_new.is_owned());
     }
 
     #[test]
@@ -163,21 +152,21 @@ mod test {
         let src_vec = NONASCII_STR
             .iter()
             .copied()
-            .map(String::from)
+            .map(Box::<str>::from)
             .collect::<Vec<_>>();
-        let mut zerovec: VarZeroVec<String> =
-            VarZeroVec::try_from_bytes(NONASCII_BYTES).expect("parse");
+        let mut zerovec: VarZeroVec<str> =
+            VarZeroVec::parse_byte_slice(NONASCII_BYTES).expect("parse");
         assert_eq!(zerovec.to_vec(), src_vec);
         let bincode_buf = bincode::serialize(&zerovec).expect("serialize");
         let zerovec_result =
-            bincode::deserialize::<VarZeroVec<String>>(&bincode_buf).expect("deserialize");
+            bincode::deserialize::<VarZeroVec<str>>(&bincode_buf).expect("deserialize");
         assert_eq!(zerovec_result.to_vec(), src_vec);
 
         // try again with owned zerovec
         zerovec.make_mut();
         let bincode_buf = bincode::serialize(&zerovec).expect("serialize");
         let zerovec_result =
-            bincode::deserialize::<VarZeroVec<String>>(&bincode_buf).expect("deserialize");
+            bincode::deserialize::<VarZeroVec<str>>(&bincode_buf).expect("deserialize");
         assert_eq!(zerovec_result.to_vec(), src_vec);
     }
 }
