@@ -4,10 +4,11 @@
 
 use super::ZeroVec;
 use crate::ule::*;
+use alloc::vec::Vec;
+use core::fmt;
+use core::marker::PhantomData;
 use serde::de::{self, Deserialize, Deserializer, SeqAccess, Visitor};
 use serde::ser::{Serialize, SerializeSeq, Serializer};
-use std::fmt;
-use std::marker::PhantomData;
 
 struct ZeroVecVisitor<T> {
     marker: PhantomData<fn() -> T>,
@@ -24,7 +25,6 @@ impl<T> Default for ZeroVecVisitor<T> {
 impl<'de, T> Visitor<'de> for ZeroVecVisitor<T>
 where
     T: 'de + Deserialize<'de> + AsULE,
-    <<T as AsULE>::ULE as ULE>::Error: fmt::Display,
 {
     type Value = ZeroVec<'de, T>;
 
@@ -36,7 +36,7 @@ where
     where
         E: de::Error,
     {
-        ZeroVec::try_from_bytes(bytes).map_err(de::Error::custom)
+        ZeroVec::parse_byte_slice(bytes).map_err(de::Error::custom)
     }
 
     fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
@@ -49,7 +49,7 @@ where
             Vec::new()
         };
         while let Some(value) = seq.next_element::<T>()? {
-            vec.push(T::as_unaligned(&value));
+            vec.push(T::as_unaligned(value));
         }
         Ok(ZeroVec::Owned(vec))
     }
@@ -59,7 +59,6 @@ where
 impl<'de, 'a, T> Deserialize<'de> for ZeroVec<'a, T>
 where
     T: 'de + Deserialize<'de> + AsULE,
-    <<T as AsULE>::ULE as ULE>::Error: fmt::Display,
     'de: 'a,
 {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
@@ -78,7 +77,7 @@ where
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
 impl<T> Serialize for ZeroVec<'_, T>
 where
-    T: Serialize + AsULE + Copy,
+    T: Serialize + AsULE,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
